import random
from itertools import combinations
from pathlib import Path

from smarts.sstudio import gen_scenario
from smarts.sstudio.types import (
    Flow,
    Route,
    Scenario,
    Traffic,
    TrafficActor,
    EndlessMission,
    Mission,
    Via,
    SocialAgentActor,
    Distribution,
)

normal = TrafficActor(name="leader", speed=Distribution(sigma=0.5, mean=1))

# flow_name = (start_lane, end_lane,)
route_opt = [
    (0, 0),
    (1, 1),
]

# Traffic combinations = 3C2 + 3C3 = 3 + 1 = 4
# Repeated traffic combinations = 4 * 100 = 400
min_flows = 2
max_flows = 2
route_comb = [
    com
    for elems in range(min_flows, max_flows + 1)
    for com in combinations(route_opt, elems)
] * 10

traffic = {}
for name, routes in enumerate(route_comb):
    traffic[str(name)] = Traffic(
        engine="SUMO",
        flows=[
            Flow(
                route=Route(
                    begin=("E0", r[0], 20),
                    end=("E0", r[1], "max"),
                ),
                # Random flow rate, between x and y vehicles per minute.
                rate=60 * random.uniform(5, 10),
                # Random flow start time, between x and y seconds.
                begin=random.uniform(0, 5),
                # For an episode with maximum_episode_steps=3000 and step
                # time=0.1s, maximum episode time=300s. Hence, traffic set to
                # end at 900s, which is greater than maximum episode time of
                # 300s.
                end=60 * 15,
                actors={normal: 1},
                randomly_spaced=True,
            )
            for r in routes
        ],
    )


ego_missions = [
    EndlessMission(
        begin=("E0", 1, 5), start_time=10
    )  # Delayed start, to ensure road has prior traffic.
]


leader_mission = [
    Mission(
        route=Route(
            begin=("E0", 1, 13),
            end=("E0", 0, "max"),
        ),
        via=(
            Via(
                "E0",
                lane_offset=100,
                lane_index=0,
                required_speed=15,
            ),
            Via(
                "E0",
                lane_offset=200,
                lane_index=1,
                required_speed=20,
            ),
            Via(
                "E0",
                lane_offset=250,
                lane_index=0,
                required_speed=7,
            ),
        ),
        start_time=10,
    ),
]


leader_actor = [
    SocialAgentActor(
        name="agent-007",
        agent_locator="zoo.policies:chase-via-points-agent-v0",
    )
]


gen_scenario(
    scenario=Scenario(
        traffic=traffic,
        social_agent_missions={"leading": (leader_actor, leader_mission)},
        ego_missions=ego_missions,
    ),
    output_dir=Path(__file__).parent,
)
